Attribute VB_Name = "ini"
Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long
Option Explicit

Public Sub InsureKeyIsNotBlank(sSection$, sKey$, sDefault$, Optional sIniFile$)
'Insures an INI file key value is non-blank. A key will be created with the
'value specified in sDefault$ if the key does not exist or is blank. For example,
'"wallpaper=" will be changed to "wallpaper=brick.bmp" (if sDefault is "brick.bmp").
'This sub is used when setting up INI files at program initialization.
Dim sValue As String
Dim lLength As Long

    If sIniFile = "" Then sIniFile = IniFileName()
    sValue = String$(255, 0)
    lLength = GetPrivateProfileString(sSection, sKey, "", sValue, 255, sIniFile)
    If lLength < 1 Then
        sValue = ""
    Else
        sValue = Left$(sValue, lLength)
    End If
    If sValue = "" Then
        WritePrivateProfileString sSection, sKey, sDefault, sIniFile
    End If
    
End Sub


Public Sub InsureKeyExists(sSection$, sKey$, sValue$, Optional sIniFile$)
'Insures an INI file key exists. A key will be created with the value
'specified in sValue if the key does not already exist. A blank key will
'be left alone. For example, "wallpaper=" will be untouched. This
'is used when setting up INI files at program initialization.
    
    If sIniFile = "" Then sIniFile = IniFileName()
    If Not KeyExists(sSection, sKey, sIniFile) Then
        WritePrivateProfileString sSection, sKey, sValue, sIniFile
    End If
    
End Sub



Public Function KeyExists(sSectionName$, sKeyName$, Optional sIniFile$) As Boolean
Dim lReturnValue As Long
Dim sKeys As String
    
    If sIniFile = "" Then sIniFile = IniFileName()
    sKeys = String$(32767, 0)
    lReturnValue = GetPrivateProfileString(sSectionName$, vbNullString, "", sKeys, 32767, sIniFile)
    sKeys = Left$(sKeys, lReturnValue)
    If InStr(sKeys, vbNullChar & sKeyName & vbNullChar) = 0 Then
        If InStr(sKeys, sKeyName & vbNullChar) = 1 Then
            KeyExists = True
        Else
            KeyExists = False
        End If
    Else
        KeyExists = True
    End If
End Function

Function IniFileName() As String
Dim sIniFile As String
Dim lLength As Long
Dim sValue As String

    'Get the base private ini file name
    If Right$(App.Path, 1) = "\" Then
        sIniFile = App.Path
    Else
        sIniFile = App.Path & "\"
    End If
    sIniFile = sIniFile & LCase$(App.EXEName) & ".ini"
    'Check to see if it exists before going further
    If Dir$(sIniFile) = "" Then
        IniFileName = sIniFile
        Exit Function
    End If
    'Check for shared ini file under [Program] IniFile=
    sValue = String$(255, 0)
    lLength = GetPrivateProfileString(LCase(App.EXEName), "IniFile", "", sValue, 255, sIniFile)
    If lLength > 0 Then sIniFile = Left$(sValue, lLength)
    IniFileName = sIniFile
    
End Function


Public Sub InitializeIniFile()
Dim lLength As Long
Dim sIniFile As String
Dim sProgramName As String

    sProgramName = App.Path & "\" & LCase(App.EXEName) & ".exe"
    sIniFile = IniFileName()
    'Create the INI file if it doesn't exist by writing one entry
    WritePrivateProfileString LCase(App.EXEName), "Program", sProgramName, sIniFile
    'Create a reference to a shared INI file
    InsureKeyIsNotBlank LCase(App.EXEName), "IniFile", sIniFile
    'Create the shared INI file if it doesn't exist
    WritePrivateProfileString LCase(App.EXEName), "Program", sProgramName, sIniFile
    'Create the log file
    InsureKeyExists LCase(App.EXEName), "LogFile", App.Path & "\" & LCase(App.EXEName) & ".log"

End Sub

Public Function SetGetIni(sSection$, sKey$, sDefault$, Optional sIniFile$) As String
Dim sValue As String
Dim lLength As Long

    If sIniFile = "" Then sIniFile = IniFileName()
    sValue = String$(255, 0)
    lLength = GetPrivateProfileString(sSection, sKey, "", sValue, 255, sIniFile)
    If lLength < 1 Then
        sValue = ""
    Else
        sValue = Left$(sValue, lLength)
    End If
    If sValue = "" Then
        lLength = WritePrivateProfileString(sSection, sKey, sDefault, sIniFile)
        sValue = sDefault
    End If
    SetGetIni = sValue
    
End Function

Public Function GetIni(sSection$, sKey$, Optional sIniFile$)
Dim sValue As String
Dim lLength As Long

    If sIniFile = "" Then sIniFile = IniFileName()
    sValue = String$(255, 0)
    lLength = GetPrivateProfileString(sSection, sKey, "", sValue, 255, sIniFile)
    If lLength < 1 Then
        GetIni = ""
    Else
        GetIni = Left$(sValue, lLength)
    End If
End Function

Public Function SectionExists(sSectionName$, Optional sIniFile$) As Boolean
Dim lReturnValue As Long
Dim sSections As String
    
    If sIniFile = "" Then sIniFile = IniFileName()
    sSections = String$(32767, 0)
    lReturnValue = GetPrivateProfileString(vbNullString, "", "", sSections, 32767, sIniFile)
    sSections = Left$(sSections, lReturnValue)
    
    If InStr(sSections, vbNullChar & sSectionName & vbNullChar) = 0 Then
        If InStr(sSections, sSectionName & vbNullChar) = 1 Then
            SectionExists = True
        Else
            SectionExists = False
        End If
    Else
        SectionExists = True
    End If
End Function
Sub SetIni(sSection$, sKey$, sValue$, Optional sIniFile$)
Dim lLength As Long

    If sIniFile = "" Then sIniFile = IniFileName()
    'Actually write the value
    lLength = WritePrivateProfileString(sSection, sKey, sValue, sIniFile)

End Sub


